# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Configure the DRAC driver in Ironic
#
# === Parameters
#
# [*package_ensure*]
#   (optional) The state of the dracclient package
#   Defaults to 'present'
#

class ironic::drivers::drac (
  $package_ensure = 'present',
) {

  ensure_packages('python-dracclient',
    {
      ensure => $package_ensure,
      tag    => ['openstack', 'ironic-package'],
    }
  )

}
