#
# Copyright (C) 2013 eNovance SAS <licensing@enovance.com>
#
# Author: Emilien Macchi <emilien.macchi@enovance.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
# Unit tests for ironic::client
#

require 'spec_helper'

describe 'ironic::client' do

  context 'on Debian platforms' do
    let :facts do
       @default_facts.merge({ :osfamily => 'Debian' })
    end

    it { is_expected.to contain_class('ironic::client') }
  end

  context 'on RedHat platforms' do
    let :facts do
       @default_facts.merge({ :osfamily => 'RedHat' })
    end

    it { is_expected.to contain_class('ironic::client') }
  end
end
